﻿using UnityEngine;
using UnityEngine.UI;
using UnityEngine.Networking;

public class PlayerController : NetworkBehaviour
{
    public Text uiText;

    public AudioClip[] sfx;
    public AudioSource audioSource;

    public AudioClip[] easterEgg;
    public AudioClip easterEggActivateClip;
    public AudioClip easterEggDeactivateClip;
    public AudioClip fatality;
    public AudioClip winPointSound;
    public AudioClip losePointSound;

    public Sprite lightsaberBackground;
    public Sprite normalBackground;
    public Image bgImage;

    public AudioSource backgroundSource;

    private AudioClip[] normalSfx;

    float lowPassFilterFactor;
    Vector3 lowPassValue;
    float accelerometerUpdateInterval = 1.0f / 60.0f;
    float lowPassKernelWidthInSeconds = 2.5f;
    float shakeDetectionThreshold = 2.5f;
    float forceToSpeed = 4f;

    float cooldownTime = 0.35f;
    float timeToHit = 2f;
    bool myTurn = false;
    static public uint previousPlayerID;
    static private bool easterEggActive = false;
    static private bool enemyEasterEggActive = false;

    float winLoseTime = 5f;

    //private int round = 0;
    //private int[] serveOrder = new int[]{ 1,1,2,2,1,1,2,2,1,1,2,2,1,1 };

    public enum EStates
    {
        WaitingForAllPlayers,
        Serving,
        Round,
        ScorePoint,
    }

    static EStates state;

    public override void OnStartServer()
    {
        base.OnStartServer();

        s1.score = s2.score = 0;
    }

    public override void OnStartLocalPlayer()
    {
        CmdConnectPlayer(netId.Value);
    }

    public override void OnStartClient()
    {
        base.OnStartClient();
        uiText = GameObject.FindGameObjectWithTag("Score").GetComponent<Text>();

        normalSfx = sfx;
    }

    [Command]
    public void CmdConnectPlayer(uint netID)
    {
        if(s1.netID == 0)
        {
            s1.netID = netID;
        }
        else if(s2.netID == 0)
        {
            s2.netID = netID;
        }

        if(NetworkServer.connections.Count == 2)
        {
            RpcSetGameState(EStates.Serving);
        }
    }

    [ClientRpc]
    public void RpcSetGameState(EStates newState)
    {
        Debug.Log("RPC new state: " + newState);
        state = newState;

        if(state == EStates.Serving)
        {
            GameObject go = GameObject.FindGameObjectWithTag("WaitingPanel");
            if(go != null)
                go.SetActive(false);
        }
        else if(state == EStates.ScorePoint)
        {
            winLoseTime = 5f;
        }
    }

    private void Start()
    {
        lowPassFilterFactor = accelerometerUpdateInterval / lowPassKernelWidthInSeconds;
        shakeDetectionThreshold *= shakeDetectionThreshold;
        lowPassValue = Input.acceleration;
        bgImage = GameObject.FindGameObjectWithTag("BgImage").GetComponent<Image>();
        name = netId.Value.ToString();
       // lightsaberBackground.
    }
    
    void Update()
    {
        if(!isLocalPlayer)
        {
            return;
        }

        DebugWrite();


        cooldownTime -= Time.deltaTime;

        myTurn = previousPlayerID != netId.Value;

        if(!easterEggActive && SwipeManager.IsSwipingUp())
        {
            audioSource.PlayOneShot(easterEggActivateClip);
            backgroundSource.Play();
            easterEggActive = true;
            bgImage.sprite = lightsaberBackground; 
            sfx = easterEgg;
        }
        else if(easterEggActive && SwipeManager.IsSwipingDown())
        {
            audioSource.PlayOneShot(easterEggDeactivateClip);
            easterEggActive = false;
            bgImage.sprite = normalBackground;
            sfx = normalSfx;
            backgroundSource.Stop();
        }

        if(!myTurn || cooldownTime > 0f || state == EStates.WaitingForAllPlayers)
        {
            return;
        }

        if(state == EStates.Round)
        {
            UpdateReceiving();
        }
        else if (state == EStates.ScorePoint)
        {
            winLoseTime -= Time.deltaTime;

            if(winLoseTime > 0f)
                return;
            else
            {
                state = EStates.Serving;
                RpcSetGameState(EStates.Serving);
            }
        }

        Vector3 acceleration = Input.acceleration;
        lowPassValue = Vector3.Lerp(lowPassValue, acceleration, lowPassFilterFactor);
        Vector3 deltaAcceleration = acceleration - lowPassValue;
        float force = deltaAcceleration.sqrMagnitude;

        if(force >= shakeDetectionThreshold || Application.isEditor && Input.GetMouseButtonDown(0))
        {
            Swing(force);
        }
    }

    //local player made a hit and plays a sound
    void Swing(float force)
    {
        audioSource.PlayOneShot(sfx[Random.Range(0, sfx.Length)]);
        cooldownTime = 0.5f;

        //uiText.text = force.ToString();

        CmdSwing(netId.Value, force, easterEggActive);
    }

    //server switches turns.
    [Command]
    public void CmdSwing(uint id, float force, bool lightsaber)
    {
        RpcSetGameState(EStates.Round);
        previousPlayerID = id;
        RpcReceiveBall(id, force, lightsaber);
    }

    //Server tells all clients that active player is player with netID = previosPlayerID
    [ClientRpc]
    void RpcReceiveBall(uint id, float force, bool lightsaber)
    {
        previousPlayerID = id;
        enemyEasterEggActive = lightsaber;
        ReceiveShot(force);
    }

    void ReceiveShot(float force)
    {
        Debug.Log("Received shot with force: " + force + " Time to react: " + Mathf.Lerp(0.3f, 1.7f, (forceToSpeed / force)));
        timeToHit = Mathf.Lerp(0.3f, 1.7f, (forceToSpeed / force));
    }

    void UpdateReceiving()
    {
        timeToHit -= Time.deltaTime;

        if(timeToHit <= 0f)
        {
            LosePoint();
        }
    }

    void LosePoint()
    {
        Debug.Log("lose point");
        audioSource.PlayOneShot(losePointSound);
        state = EStates.ScorePoint;
        CmdPlayerMissedShot(netId.Value, enemyEasterEggActive);
    }


    static Score s1, s2;
    [Command]
    void CmdPlayerMissedShot(uint id, bool lightsaber)
    {
        RpcSetGameState(EStates.ScorePoint);

        uint scoreDelta = 1;

        if(lightsaber)
        {
            scoreDelta = 10;
        }

        //Update scores
        if(s1.netID == id)
            s2.score += scoreDelta;
        if(s2.netID == id)
            s1.score += scoreDelta;

        

        Debug.Log("CMD new scores: " + s1.netID + ": " + s1.score + " " + s2.netID + ": " + s2.score);

        RpcSetScore(s1, s2, lightsaber);
    }

    [ClientRpc]
    void RpcNextTurn(uint id)
    {
        Debug.Log("RPC next turn");
        previousPlayerID = id;
    }

    [ClientRpc]
    void RpcSetScore(Score score1, Score score2, bool lightsaber)
    {
        Debug.Log("RPC new scores: " + score1.netID + ": " + score1.score + " " + score2.netID + ": " + score2.score);
        if(lightsaber)
        {
            audioSource.PlayOneShot(fatality);
        }
        else
        {
            audioSource.PlayOneShot(winPointSound);
        }

        if (score1.score >= 11 || score2.score >= 11)
        {
            var nm = GameObject.FindGameObjectWithTag("NetworkManager").GetComponent<CustomHUD>();
            nm.QuitButton();

            SceneManager.SwitchSceneS("MultiPlayer");
        }

        uiText.text = "Player 1: " + score1.score + "\n" + "Player 2: " + score2.score;
    }

    void DebugWrite()
    {
        string s = "";
        switch(state)
        {
            case EStates.Round:
                {
                    s = previousPlayerID != netId.Value ? "Hit!" : "Wait";
                }
                break;

            case EStates.Serving:
                {
                    s = previousPlayerID != netId.Value ? "You Serve!" : "Opponent Serves!";
                }
                break;
        }

        GameObject.FindGameObjectWithTag("State").GetComponent<Text>().text = s;


        //GameObject.FindGameObjectWithTag("CurrentPlayerID").GetComponent<Text>().text = previousPlayerID.ToString();
        //GameObject.FindGameObjectWithTag("TimeToReceive").GetComponent<Text>().text = timeToHit.ToString();
    }
}
